import { UndoManager } from "loro-crdt";
import { createCursorLayer, createSelectionLayer, LoroAwarenessPlugin, remoteAwarenessStateField, RemoteAwarenessPlugin as RemoteAwarenessPlugin, loroCursorTheme, } from "./awareness";
import { LoroSyncPluginValue } from "./sync";
import { keymap, ViewPlugin } from "@codemirror/view";
import { undoKeyMap, undoManagerStateField, UndoPluginValue } from "./undo";
export function loroExtension(doc, awareness, undoConfig) {
    let extension = [
        ViewPlugin.define((view) => new LoroSyncPluginValue(view, doc))
            .extension,
    ];
    if (undoConfig) {
        const undoManager = new UndoManager(doc, undoConfig);
        extension = extension.concat([
            undoManagerStateField.init(() => undoManager),
            keymap.of([...undoKeyMap]),
            ViewPlugin.define((view) => new UndoPluginValue(view, doc, undoManager)).extension,
        ]);
    }
    if (awareness) {
        extension = extension.concat([
            remoteAwarenessStateField,
            createCursorLayer(),
            createSelectionLayer(),
            ViewPlugin.define((view) => new LoroAwarenessPlugin(view, doc, awareness.user, awareness.awareness)),
            ViewPlugin.define((view) => new RemoteAwarenessPlugin(view, doc, awareness.awareness)),
            loroCursorTheme,
        ]);
    }
    return extension;
}
